/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>
#include <glib.h>
#include "gbcode_pinyin_table.h"

static int 
gbcode_get_character_unicode_int (const char *str)
{   
    if (str == NULL)
      return -1;

    setlocale(LC_ALL,"");
    wchar_t *h = (wchar_t *)malloc (sizeof (wchar_t));

    if (mbtowc (h, str, strlen(str)) < 0)
      return -1;

    int ret = (int)h[0];
    free (h);

    return ret;
}

static void
gbcode_substr (const char *str, int start, int length, char *ret)
{
  if (length < 1) {
    return ;
  } else {
    const char *str_tmp = str;
    const char *str_sta = str;
    unsigned int size = 0;
    gboolean flag = FALSE;  

    while (str_tmp != NULL && *str_tmp != '\0' && length > 0)
    {
      if (start <= 0) {
        flag = TRUE;
        if (start == 0)
          str_sta = str_tmp;
      }
      if (*str_tmp > 0x80 || *str_tmp < 0) {
        str_tmp += 3;
	if (flag) {
          size += 3;
          length --;
        }
      } else {
        str_tmp ++;
        if (flag) {
          size ++;
          length --;
        }
      }
      start --;
    }
    strncpy (ret, str_sta, size);
  }
}

int 
gbcode_get_str_len (const char *str)
{
  int len = 0;
  const char *string = str;

  while (string != NULL && *string != '\0') {
    if (*string > 0x80 || *string < 0) {
      string += 2;
    }
    string ++;
    len ++;
  }
  return len;
}

static void
gbcode_capitalize (char *pinyin)
{
  if (strlen (pinyin) > 0) {
    if(pinyin[0] >= 'a'&& pinyin[0] <= 'z') {
      pinyin[0] += 'A'-'a';
    }
  }
}

void
gbcode_get_full_pinyin (const char *str, gboolean capitalize, char *result, unsigned int result_size)
{
  unsigned int i = 0, len = 0, str_len = 0;
  int unicode;

  for (i = 0, str_len = gbcode_get_str_len (str); i < str_len; i++) {
    char ch[6] = {0, };
    char chars[6] = {0, };

    gbcode_substr (str, i, 1, ch);
    unicode = gbcode_get_character_unicode_int (ch);
    if (unicode > 40869 || unicode < 19968) {
      strcat (result, ch);
    } else {
      gbcode_get_pinyin_by_unicode (ch, chars);

      int tmp = strlen (chars);
      if (result_size < (len += tmp)) {
        return ;
      }

      if (tmp > 0) {
        if (capitalize)
          gbcode_capitalize (chars);
        strcat (result, chars);
      }
    }
  }
}
